@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2014 MongoDB, Inc.
+ * Copyright (c) 2008-2016 MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,10 +21,14 @@ import com.mongodb.async.FutureResultCallback;
 import com.mongodb.async.SingleResultCallback;
 import com.mongodb.binding.AsyncClusterBinding;
 import com.mongodb.binding.AsyncConnectionSource;
+import com.mongodb.binding.AsyncReadBinding;
 import com.mongodb.binding.AsyncReadWriteBinding;
+import com.mongodb.binding.AsyncSessionBinding;
 import com.mongodb.binding.AsyncSingleConnectionBinding;
+import com.mongodb.binding.AsyncWriteBinding;
 import com.mongodb.binding.ClusterBinding;
 import com.mongodb.binding.ReadWriteBinding;
+import com.mongodb.binding.SessionBinding;
 import com.mongodb.binding.SingleConnectionBinding;
 import com.mongodb.connection.AsyncConnection;
 import com.mongodb.connection.AsynchronousSocketChannelStreamFactory;
@@ -42,21 +46,27 @@ import com.mongodb.connection.SocketStreamFactory;
 import com.mongodb.connection.SslSettings;
 import com.mongodb.connection.StreamFactory;
 import com.mongodb.connection.netty.NettyStreamFactory;
-import com.mongodb.management.JMXConnectionPoolListener;
 import com.mongodb.operation.AsyncReadOperation;
 import com.mongodb.operation.AsyncWriteOperation;
+import com.mongodb.operation.BatchCursor;
 import com.mongodb.operation.CommandWriteOperation;
 import com.mongodb.operation.DropDatabaseOperation;
+import com.mongodb.operation.ReadOperation;
+import com.mongodb.operation.WriteOperation;
+import com.mongodb.selector.ServerSelector;
 import org.bson.BsonDocument;
-import org.bson.BsonDocumentWrapper;
 import org.bson.BsonInt32;
+import org.bson.BsonString;
+import org.bson.BsonValue;
 import org.bson.Document;
 import org.bson.codecs.BsonDocumentCodec;
 import org.bson.codecs.DocumentCodec;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import static com.mongodb.connection.ClusterConnectionMode.MULTIPLE;
 import static com.mongodb.connection.ClusterType.REPLICA_SET;
@@ -67,6 +77,7 @@ import static java.util.Arrays.asList;
 import static java.util.concurrent.TimeUnit.SECONDS;
 import static org.hamcrest.CoreMatchers.is;
 import static org.junit.Assume.assumeThat;
+import static org.junit.Assume.assumeTrue;
 
 /**
  * Helper class for the acceptance tests.  Used primarily by DatabaseTestCase and FunctionalSpecification.  This fixture allows Test
@@ -76,11 +87,14 @@ public final class ClusterFixture {
     public static final String DEFAULT_URI = "mongodb://localhost:27017";
     public static final String MONGODB_URI_SYSTEM_PROPERTY_NAME = "org.mongodb.test.uri";
     private static final String DEFAULT_DATABASE_NAME = "JavaDriverTest";
+    private static final int COMMAND_NOT_FOUND_ERROR_CODE = 59;
     public static final long TIMEOUT = 60L;
 
     private static ConnectionString connectionString;
     private static Cluster cluster;
     private static Cluster asyncCluster;
+    private static Map<ReadPreference, ReadWriteBinding> bindingMap = new HashMap<ReadPreference, ReadWriteBinding>();
+    private static Map<ReadPreference, AsyncReadWriteBinding> asyncBindingMap = new HashMap<ReadPreference, AsyncReadWriteBinding>();
 
     static {
         String mongoURIProperty = System.getProperty(MONGODB_URI_SYSTEM_PROPERTY_NAME);
@@ -101,13 +115,41 @@ public final class ClusterFixture {
         return getCluster().getDescription().getType() == clusterType;
     }
 
-    @SuppressWarnings("deprecation")
     public static ServerVersion getServerVersion() {
-        return getCluster().getDescription().getAny().get(0).getVersion();
+        return getCluster().selectServer(new ServerSelector() {
+            @Override
+            @SuppressWarnings("deprecation")
+            public List<ServerDescription> select(final ClusterDescription clusterDescription) {
+                return clusterDescription.getAny();
+            }
+        }).getDescription().getVersion();
     }
 
     public static boolean serverVersionAtLeast(final List<Integer> versionArray) {
-        return getConnectedServerVersion().compareTo(new ServerVersion(versionArray)) >= 0;
+        return getServerVersion().compareTo(new ServerVersion(versionArray)) >= 0;
+    }
+
+    public static boolean serverVersionAtLeast(final int majorVersion, final int minorVersion) {
+        return serverVersionAtLeast(asList(majorVersion, minorVersion, 0));
+    }
+
+    public static boolean serverVersionLessThan(final String versionString) {
+        return getServerVersion().compareTo(new ServerVersion(getVersionList(versionString).subList(0, 3))) < 0;
+    }
+
+    public static boolean serverVersionGreaterThan(final String versionString) {
+        return getServerVersion().compareTo(new ServerVersion(getVersionList(versionString).subList(0, 3))) > 0;
+    }
+
+    private static List<Integer> getVersionList(final String versionString) {
+        List<Integer> versionList = new ArrayList<Integer>();
+        for (String s : versionString.split("\\.")) {
+            versionList.add(Integer.valueOf(s));
+        }
+        while (versionList.size() < 3) {
+            versionList.add(0);
+        }
+        return versionList;
     }
 
     public static Document getBuildInfo() {
@@ -138,30 +180,23 @@ public final class ClusterFixture {
         return storageEngine != null && !storageEngine.get("name").equals("inMemory");
     }
 
-    @SuppressWarnings("deprecation")
-    private static ServerVersion getConnectedServerVersion() {
-        ClusterDescription clusterDescription = getCluster().getDescription();
-        int retries = 0;
-        while (clusterDescription.getAny().isEmpty() && retries <= 3) {
-            try {
-                Thread.sleep(1000);
-                retries++;
-            } catch (InterruptedException e) {
-                throw new RuntimeException("Interrupted", e);
-            }
-            clusterDescription = getCluster().getDescription();
-        }
-        if (clusterDescription.getAny().isEmpty()) {
-            throw new RuntimeException("There are no servers available in " + clusterDescription);
-        }
-        return clusterDescription.getAny().get(0).getVersion();
+    public static boolean isNotAtLeastJava7() {
+        return javaVersionStartsWith("1.6");
+    }
+
+    public static boolean isNotAtLeastJava8() {
+        return isNotAtLeastJava7() || javaVersionStartsWith("1.7");
+    }
+
+    private static boolean javaVersionStartsWith(final String versionPrefix) {
+        return System.getProperty("java.version").startsWith(versionPrefix + ".");
     }
 
     static class ShutdownHook extends Thread {
         @Override
         public void run() {
             if (cluster != null) {
-                new DropDatabaseOperation(getDefaultDatabaseName()).execute(getBinding());
+                new DropDatabaseOperation(getDefaultDatabaseName(), WriteConcern.ACKNOWLEDGED).execute(getBinding());
                 cluster.close();
             }
         }
@@ -171,20 +206,27 @@ public final class ClusterFixture {
         return connectionString;
     }
 
+    public static ReadWriteBinding getBinding(final Cluster cluster) {
+        return new ClusterBinding(cluster, ReadPreference.primary());
+    }
+
     public static ReadWriteBinding getBinding() {
-        return getBinding(getCluster());
+        return getBinding(getCluster(), ReadPreference.primary());
     }
 
     public static ReadWriteBinding getBinding(final ReadPreference readPreference) {
         return getBinding(getCluster(), readPreference);
     }
 
-    public static ReadWriteBinding getBinding(final Cluster cluster) {
-        return getBinding(cluster, ReadPreference.primary());
-    }
-
     private static ReadWriteBinding getBinding(final Cluster cluster, final ReadPreference readPreference) {
-        return new ClusterBinding(cluster, readPreference);
+        if (!bindingMap.containsKey(readPreference)) {
+            ReadWriteBinding binding = new ClusterBinding(cluster, readPreference);
+            if (serverVersionAtLeast(3, 6)) {
+                binding = new SessionBinding(binding);
+            }
+            bindingMap.put(readPreference, binding);
+        }
+        return bindingMap.get(readPreference);
     }
 
     public static SingleConnectionBinding getSingleConnectionBinding() {
@@ -199,20 +241,27 @@ public final class ClusterFixture {
         return new AsyncSingleConnectionBinding(cluster, 20, SECONDS);
     }
 
+    public static AsyncReadWriteBinding getAsyncBinding(final Cluster cluster) {
+        return new AsyncClusterBinding(cluster, ReadPreference.primary());
+    }
+
     public static AsyncReadWriteBinding getAsyncBinding() {
-        return getAsyncBinding(getAsyncCluster());
+        return getAsyncBinding(getAsyncCluster(), ReadPreference.primary());
     }
 
     public static AsyncReadWriteBinding getAsyncBinding(final ReadPreference readPreference) {
         return getAsyncBinding(getAsyncCluster(), readPreference);
     }
 
-    public static AsyncReadWriteBinding getAsyncBinding(final Cluster cluster) {
-        return getAsyncBinding(cluster, ReadPreference.primary());
-    }
-
     public static AsyncReadWriteBinding getAsyncBinding(final Cluster cluster, final ReadPreference readPreference) {
-        return new AsyncClusterBinding(cluster, readPreference);
+        if (!asyncBindingMap.containsKey(readPreference)) {
+            AsyncReadWriteBinding binding = new AsyncClusterBinding(cluster, readPreference);
+            if (serverVersionAtLeast(3, 6)) {
+                binding = new AsyncSessionBinding(binding);
+            }
+            asyncBindingMap.put(readPreference, binding);
+        }
+        return asyncBindingMap.get(readPreference);
     }
 
     public static synchronized Cluster getCluster() {
@@ -229,14 +278,15 @@ public final class ClusterFixture {
         return asyncCluster;
     }
 
+    @SuppressWarnings("deprecation")
     public static Cluster createCluster(final StreamFactory streamFactory) {
-        return new DefaultClusterFactory().create(ClusterSettings.builder().applyConnectionString(getConnectionString()).build(),
+        return new DefaultClusterFactory().createCluster(ClusterSettings.builder().applyConnectionString(getConnectionString()).build(),
                                                   ServerSettings.builder().build(),
                                                   ConnectionPoolSettings.builder().applyConnectionString(getConnectionString()).build(),
                                                   streamFactory,
                                                   new SocketStreamFactory(SocketSettings.builder().build(), getSslSettings()),
-                                                  getConnectionString().getCredentialList(), null, new JMXConnectionPoolListener(), null);
-
+                                                  getConnectionString().getCredentialList(), null, null, null,
+                                                  getConnectionString().getCompressorList());
     }
 
     public static StreamFactory getAsyncStreamFactory() {
@@ -256,7 +306,11 @@ public final class ClusterFixture {
     }
 
     public static SslSettings getSslSettings() {
-        SslSettings.Builder builder = SslSettings.builder().applyConnectionString(getConnectionString());
+        return getSslSettings(getConnectionString());
+    }
+
+    public static SslSettings getSslSettings(final ConnectionString connectionString) {
+        SslSettings.Builder builder = SslSettings.builder().applyConnectionString(connectionString);
         if (System.getProperty("java.version").startsWith("1.6.")) {
             builder.invalidHostNameAllowed(true);
         }
@@ -264,15 +318,34 @@ public final class ClusterFixture {
     }
 
     @SuppressWarnings("deprecation")
-    public static ServerAddress getPrimary() throws InterruptedException {
+    public static ServerAddress getPrimary() {
         List<ServerDescription> serverDescriptions = getCluster().getDescription().getPrimaries();
         while (serverDescriptions.isEmpty()) {
-            sleep(100);
+            try {
+                sleep(100);
+            } catch (InterruptedException e) {
+                throw new RuntimeException(e);
+            }
             serverDescriptions = getCluster().getDescription().getPrimaries();
         }
         return serverDescriptions.get(0).getAddress();
     }
 
+    @SuppressWarnings("deprecation")
+    public static ServerAddress getSecondary() {
+        List<ServerDescription> serverDescriptions = getCluster().getDescription().getSecondaries();
+        while (serverDescriptions.isEmpty()) {
+            try {
+                sleep(100);
+            } catch (InterruptedException e) {
+                throw new RuntimeException(e);
+            }
+            serverDescriptions = getCluster().getDescription().getSecondaries();
+        }
+        return serverDescriptions.get(0).getAddress();
+    }
+
+    @SuppressWarnings("deprecation")
     public static List<MongoCredential> getCredentialList() {
         return getConnectionString().getCredentialList();
     }
@@ -291,37 +364,100 @@ public final class ClusterFixture {
     }
 
     public static boolean isAuthenticated() {
-        return !getConnectionString().getCredentialList().isEmpty();
+        return getConnectionString().getCredential() != null;
     }
 
     public static void enableMaxTimeFailPoint() {
         assumeThat(isSharded(), is(false));
-        new CommandWriteOperation<BsonDocument>("admin",
-                                                new BsonDocumentWrapper<Document>(new Document("configureFailPoint", "maxTimeAlwaysTimeOut")
-                                                                                  .append("mode", "alwaysOn"),
-                                                                                  new DocumentCodec()),
-                                                new BsonDocumentCodec())
-        .execute(getBinding());
+        boolean failsPointsSupported = true;
+        try {
+            new CommandWriteOperation<BsonDocument>("admin",
+                    new BsonDocument("configureFailPoint", new BsonString("maxTimeAlwaysTimeOut"))
+                            .append("mode", new BsonString("alwaysOn")),
+                    new BsonDocumentCodec())
+                    .execute(getBinding());
+        } catch (MongoCommandException e) {
+            if (e.getErrorCode() == COMMAND_NOT_FOUND_ERROR_CODE) {
+                failsPointsSupported = false;
+            }
+        }
+        assumeTrue("configureFailPoint is not enabled", failsPointsSupported);
     }
 
     public static void disableMaxTimeFailPoint() {
         assumeThat(isSharded(), is(false));
-        if (serverVersionAtLeast(asList(2, 6, 0)) && !isSharded()) {
-            new CommandWriteOperation<BsonDocument>("admin",
-                                                    new BsonDocumentWrapper<Document>(new Document("configureFailPoint",
-                                                                                                   "maxTimeAlwaysTimeOut")
-                                                                                      .append("mode", "off"),
-                                                                                      new DocumentCodec()),
-                                                    new BsonDocumentCodec())
-            .execute(getBinding());
+        if (!isSharded()) {
+            try {
+                new CommandWriteOperation<BsonDocument>("admin",
+                        new BsonDocument("configureFailPoint",
+                                new BsonString("maxTimeAlwaysTimeOut"))
+                                .append("mode", new BsonString("off")),
+                        new BsonDocumentCodec())
+                        .execute(getBinding());
+            } catch (MongoCommandException e) {
+                // ignore
+            }
         }
     }
 
+    public static void enableOnPrimaryTransactionalWriteFailPoint(final BsonValue failPointData) {
+        BsonDocument command = BsonDocument.parse("{ configureFailPoint: 'onPrimaryTransactionalWrite'}");
+
+        if (failPointData.isDocument() && failPointData.asDocument().containsKey("mode")) {
+            for (Map.Entry<String, BsonValue> keyValue : failPointData.asDocument().entrySet()) {
+                command.append(keyValue.getKey(), keyValue.getValue());
+            }
+        } else {
+            command.append("mode", failPointData);
+        }
+        boolean failsPointsSupported = true;
+        try {
+            new CommandWriteOperation<BsonDocument>("admin", command, new BsonDocumentCodec()).execute(getBinding());
+        } catch (MongoCommandException e) {
+            if (e.getErrorCode() == COMMAND_NOT_FOUND_ERROR_CODE) {
+                failsPointsSupported = false;
+            }
+        }
+        assumeTrue("configureFailPoint is not enabled", failsPointsSupported);
+    }
+
+    public static void disableOnPrimaryTransactionalWriteFailPoint() {
+        assumeThat(isSharded(), is(false));
+        if (!isSharded()) {
+            try {
+                new CommandWriteOperation<BsonDocument>("admin",
+                        new BsonDocument("configureFailPoint",
+                                new BsonString("onPrimaryTransactionalWrite"))
+                                .append("mode", new BsonString("off")),
+                        new BsonDocumentCodec())
+                        .execute(getBinding());
+            } catch (MongoCommandException e) {
+                // ignore
+            }
+        }
+    }
+
+    public static <T> T executeSync(final WriteOperation<T> op) throws Throwable {
+        return executeSync(op, getBinding());
+    }
+
+    public static <T> T executeSync(final WriteOperation<T> op, final ReadWriteBinding binding) throws Throwable {
+        return op.execute(binding);
+    }
+
+    public static <T> T executeSync(final ReadOperation<T> op) throws Throwable {
+        return executeSync(op, getBinding());
+    }
+
+    public static <T> T executeSync(final ReadOperation<T> op, final ReadWriteBinding binding) throws Throwable {
+        return op.execute(binding);
+    }
+
     public static <T> T executeAsync(final AsyncWriteOperation<T> op) throws Throwable {
         return executeAsync(op, getAsyncBinding());
     }
 
-    public static <T> T executeAsync(final AsyncWriteOperation<T> op, final AsyncReadWriteBinding binding) throws Throwable {
+    public static <T> T executeAsync(final AsyncWriteOperation<T> op, final AsyncWriteBinding binding) throws Throwable {
         final FutureResultCallback<T> futureResultCallback = new FutureResultCallback<T>();
         op.executeAsync(binding, futureResultCallback);
         return futureResultCallback.get(TIMEOUT, SECONDS);
@@ -331,7 +467,7 @@ public final class ClusterFixture {
         return executeAsync(op, getAsyncBinding());
     }
 
-    public static <T> T executeAsync(final AsyncReadOperation<T> op, final AsyncReadWriteBinding binding) throws Throwable {
+    public static <T> T executeAsync(final AsyncReadOperation<T> op, final AsyncReadBinding binding) throws Throwable {
         final FutureResultCallback<T> futureResultCallback = new FutureResultCallback<T>();
         op.executeAsync(binding, futureResultCallback);
         return futureResultCallback.get(TIMEOUT, SECONDS);
@@ -378,6 +514,27 @@ public final class ClusterFixture {
         });
     }
 
+    public static <T> List<T> collectCursorResults(final AsyncBatchCursor<T> batchCursor) throws Throwable {
+        final List<T> results = new ArrayList<T>();
+        FutureResultCallback<Void> futureResultCallback = new FutureResultCallback<Void>();
+        loopCursor(batchCursor, new Block<T>() {
+            @Override
+            public void apply(final T t) {
+                results.add(t);
+            }
+        }, futureResultCallback);
+        futureResultCallback.get(TIMEOUT, SECONDS);
+        return results;
+    }
+
+    public static <T> List<T> collectCursorResults(final BatchCursor<T> batchCursor) throws Throwable {
+        List<T> results = new ArrayList<T>();
+        while (batchCursor.hasNext()) {
+            results.addAll(batchCursor.next());
+        }
+        return results;
+    }
+
     public static AsyncConnectionSource getWriteConnectionSource(final AsyncReadWriteBinding binding) throws Throwable {
         final FutureResultCallback<AsyncConnectionSource> futureResultCallback = new FutureResultCallback<AsyncConnectionSource>();
         binding.getWriteConnectionSource(futureResultCallback);
