@@ -115,6 +115,16 @@ public class DBCollectionTest extends DatabaseTestCase {
         assertEquals(0, collection.getOptions());
     }
 
+    @Test
+    public void insertEmptyListShouldThrowIllegalArgumentException() {
+        try {
+            collection.insert(Collections.<DBObject>emptyList());
+            fail("Should throw IllegalArgumentException");
+        } catch (IllegalArgumentException e) {
+           // empty
+        }
+    }
+
     @Test
     public void testInsert() {
         WriteResult res = collection.insert(new BasicDBObject("_id", 1).append("x", 2));
@@ -123,6 +133,12 @@ public class DBCollectionTest extends DatabaseTestCase {
         assertEquals(new BasicDBObject("_id", 1).append("x", 2), collection.findOne());
     }
 
+    @Test
+    public void testFindWithNullQuery() {
+        collection.insert(new BasicDBObject("_id", 1).append("x", 2));
+        assertEquals(new BasicDBObject("_id", 1).append("x", 2), collection.find(null).next());
+    }
+
     @Test
     public void testInsertDuplicateKeyException() {
         DBObject doc = new BasicDBObject("_id", 1);
@@ -228,7 +244,6 @@ public class DBCollectionTest extends DatabaseTestCase {
 
     @Test(expected = MongoCommandException.class)
     public void testCreateIndexWithInvalidIndexType() {
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
         DBObject index = new BasicDBObject("x", "funny");
         collection.createIndex(index);
     }
@@ -288,8 +303,6 @@ public class DBCollectionTest extends DatabaseTestCase {
 
     @Test
     public void testCreateIndexAs2dsphere() {
-        assumeThat(serverVersionAtLeast(asList(2, 4, 0)), is(true));
-
         // when
         DBObject index = new BasicDBObject("x", "2dsphere");
         collection.createIndex(index);
@@ -301,7 +314,6 @@ public class DBCollectionTest extends DatabaseTestCase {
 
     @Test
     public void testCreateIndexAsText() {
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
         DBObject index = new BasicDBObject("x", "text");
         collection.createIndex(index);
 
@@ -429,7 +441,6 @@ public class DBCollectionTest extends DatabaseTestCase {
     @Test(expected = MongoExecutionTimeoutException.class)
     public void testFindAndUpdateTimeout() {
         assumeThat(ClusterFixture.isAuthenticated(), is(false));
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
         collection.insert(new BasicDBObject("_id", 1));
         enableMaxTimeFailPoint();
         try {
@@ -443,7 +454,6 @@ public class DBCollectionTest extends DatabaseTestCase {
     @Test(expected = MongoExecutionTimeoutException.class)
     public void testFindAndReplaceTimeout() {
         assumeThat(isSharded(), is(false));
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
         collection.insert(new BasicDBObject("_id", 1));
         enableMaxTimeFailPoint();
         try {
@@ -457,7 +467,6 @@ public class DBCollectionTest extends DatabaseTestCase {
     @Test(expected = MongoExecutionTimeoutException.class)
     public void testFindAndRemoveTimeout() {
         assumeThat(isSharded(), is(false));
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
         collection.insert(new BasicDBObject("_id", 1));
         enableMaxTimeFailPoint();
         try {
@@ -860,7 +869,7 @@ public class DBCollectionTest extends DatabaseTestCase {
 
     @Test
     public void testWriteConcernExceptionOnFindAndModify() throws UnknownHostException {
-        assumeThat(serverVersionAtLeast(asList(3, 2, 0)), is(true));
+        assumeThat(serverVersionAtLeast(3, 2), is(true));
         assumeThat(isDiscoverableReplicaSet(), is(true));
 
         ObjectId id = new ObjectId();
@@ -959,7 +968,6 @@ public class DBCollectionTest extends DatabaseTestCase {
     @Category(Slow.class)
     public void testParallelScan() throws UnknownHostException {
         assumeThat(isSharded(), is(false));
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
 
         Set<Integer> ids = new HashSet<Integer>();
         List<BasicDBObject> documents = new ArrayList<BasicDBObject>(2000);
@@ -993,7 +1001,7 @@ public class DBCollectionTest extends DatabaseTestCase {
 
         try {
             c.insert(Collections.<DBObject>singletonList(new BasicDBObject("level", 9)));
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1003,7 +1011,7 @@ public class DBCollectionTest extends DatabaseTestCase {
         try {
             c.insert(Collections.<DBObject>singletonList(new BasicDBObject("level", 9)),
                      new InsertOptions().bypassDocumentValidation(false));
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1023,7 +1031,7 @@ public class DBCollectionTest extends DatabaseTestCase {
                      new InsertOptions()
                      .bypassDocumentValidation(true)
                      .writeConcern(WriteConcern.UNACKNOWLEDGED));
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1041,7 +1049,7 @@ public class DBCollectionTest extends DatabaseTestCase {
         try {
             c.update(new BasicDBObject("_id", 1), new BasicDBObject("_id", 1).append("level", 9), true, false, WriteConcern.ACKNOWLEDGED,
                      null);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1051,7 +1059,7 @@ public class DBCollectionTest extends DatabaseTestCase {
         try {
             c.update(new BasicDBObject("_id", 1), new BasicDBObject("_id", 1).append("level", 9), true, false, WriteConcern.ACKNOWLEDGED,
                      false, null);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1077,7 +1085,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             c.update(new BasicDBObject("_id", 1), new BasicDBObject("_id", 1).append("level", 9), true, false,
                      WriteConcern.UNACKNOWLEDGED,
                      true, null);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1095,7 +1103,7 @@ public class DBCollectionTest extends DatabaseTestCase {
 
         try {
             c.findAndModify(new BasicDBObject("_id", 1), new BasicDBObject("_id", 1).append("level", 9));
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1105,7 +1113,7 @@ public class DBCollectionTest extends DatabaseTestCase {
         try {
             c.findAndModify(new BasicDBObject("_id", 1), null, null, false, new BasicDBObject("_id", 1).append("level", 9), false, false,
                             false, 0, TimeUnit.SECONDS);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1137,7 +1145,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             BulkWriteOperation bulk = c.initializeOrderedBulkOperation();
             bulk.insert(new BasicDBObject("level", 9));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1149,7 +1157,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(false);
             bulk.insert(new BasicDBObject("level", 9));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1189,7 +1197,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(true);
             bulk.insert(new BasicDBObject("level", 9));
             bulk.execute(WriteConcern.UNACKNOWLEDGED);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1207,7 +1215,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             BulkWriteOperation bulk = c.initializeOrderedBulkOperation();
             bulk.find(new BasicDBObject("_id", 1)).upsert().update(new BasicDBObject("$set", new BasicDBObject("level", 9)));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1219,7 +1227,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(false);
             bulk.find(new BasicDBObject("_id", 1)).upsert().update(new BasicDBObject("$set", new BasicDBObject("level", 9)));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1259,7 +1267,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(true);
             bulk.find(new BasicDBObject("_id", 1)).upsert().update(new BasicDBObject("$set", new BasicDBObject("level", 9)));
             bulk.execute(WriteConcern.UNACKNOWLEDGED);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1277,7 +1285,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             BulkWriteOperation bulk = c.initializeOrderedBulkOperation();
             bulk.find(new BasicDBObject("_id", 1)).upsert().replaceOne(new BasicDBObject("level", 9));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1289,7 +1297,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(false);
             bulk.find(new BasicDBObject("_id", 1)).upsert().replaceOne(new BasicDBObject("level", 9));
             bulk.execute();
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1329,7 +1337,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             bulk.setBypassDocumentValidation(true);
             bulk.find(new BasicDBObject("_id", 1)).upsert().replaceOne(new BasicDBObject("level", 9));
             bulk.execute(WriteConcern.UNACKNOWLEDGED);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1338,9 +1346,8 @@ public class DBCollectionTest extends DatabaseTestCase {
     }
 
     @Test
+    @SuppressWarnings("deprecation")
     public void testBypassDocumentValidationForAggregateDollarOut() {
-        assumeThat(serverVersionAtLeast(asList(2, 6, 0)), is(true));
-
         //given
         DBObject options = new BasicDBObject("validator", QueryBuilder.start("level").greaterThanEquals(10).get());
         DBCollection cOut = database.createCollection(collectionName + ".out", options);
@@ -1350,7 +1357,7 @@ public class DBCollectionTest extends DatabaseTestCase {
 
         try {
             c.aggregate(Collections.<DBObject>singletonList(new BasicDBObject("$out", cOut.getName())));
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1362,7 +1369,7 @@ public class DBCollectionTest extends DatabaseTestCase {
                         AggregationOptions.builder()
                         .bypassDocumentValidation(false)
                         .build());
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1403,7 +1410,7 @@ public class DBCollectionTest extends DatabaseTestCase {
             MapReduceCommand mapReduceCommand = new MapReduceCommand(c, map, reduce, cOut.getName(), MapReduceCommand.OutputType.REPLACE,
                                                                      new BasicDBObject());
             c.mapReduce(mapReduceCommand);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
@@ -1415,7 +1422,7 @@ public class DBCollectionTest extends DatabaseTestCase {
                                                                      new BasicDBObject());
             mapReduceCommand.setBypassDocumentValidation(false);
             c.mapReduce(mapReduceCommand);
-            if (serverVersionAtLeast(asList(3, 2, 0))) {
+            if (serverVersionAtLeast(3, 2)) {
                 fail();
             }
         } catch (MongoException e) {
